
/*
 * qfits example: loading pixels
 *
 * This example shows how to use the qfits pixel loader.
 * The procedure consists in:
 * 
 * - Filling up a qfitsloader structure
 * - Calling qfitsload_init() on the structure
 * - Calling qfits_loadpix() to load the pixels.
 *
 * This program takes in input the name of a FITS file, optionally an
 * extension and plane number, loads the pixels and prints out the
 * average pixel value. The pixel buffer is loaded in 3 different
 * formats (float, double, int) to demonstrate the pixel loader.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "qfits.h"
#include "xmemory.h"


int main(int argc, char * argv[])
{
    /*
     * To load pixels from a FITS image, a 'qfitsloader' object is needed.
     * The simplest is to declare it statically rather than malloc() it
     * since it is short-lived anyway and only exists for the duration of
     * the load.
     */
	qfitsloader	ql ;

    /* Following variables only useful to compute average pixel value */
    int         i, npix ;
    double      average ;

	if (argc<2) {
		printf("use: %s <filename> [plane] [extension]\n", argv[0]);
		return 1 ;
	}

	/* Set load structure */

    /* filename must be set */
	ql.filename = argv[1] ;
    /* Set xtnum and plane num to 0 if none are specified */
	ql.xtnum = 0 ;
	ql.pnum  = 0 ;
    
    /* Set the pixel load type to float for the first load */
	ql.ptype = PTYPE_FLOAT ;

	/* Get xtnum and pnum if specified on the command-line */
	if (argc>2) {
		ql.pnum = atoi(argv[2]);
	}
	if (argc>3) {
		ql.xtnum = atoi(argv[3]);
	}

    printf("Loading pixel buffer as a float array...\n");
	/* Try to initialize this load */
	if (qfitsloader_init(&ql)!=0) {
        /*
         * Something went wrong.
         * Since the error handler has been left untouched, the function
         * must have printed out something on stderr about what happened.
         * No need to be more verbose: bail out.
         */
		return -1 ;
	}
	/* Perform actual load */
	if (qfits_loadpix(&ql)!=0) {
        /*
         * Something went wrong: bailout
         */
		return -1 ;
	}

    /*
     * The pixel buffer is now loaded. Since the load was requested
     * with ptype==PTYPE_FLOAT, the buffer to examine is fbuf (float
     * pixel buffer).
     */

    /* Compute npix (number of loaded pixels) once */
    npix = ql.lx * ql.ly ;

    /* Compute the average pixel value */
    average=0 ;
    for (i=0 ; i<npix ; average += (double)ql.fbuf[i++]) ;
    average /= (double)npix ;

    /* Pixel buffer not needed any more */
	free(ql.fbuf);
	printf("Average pixel value (float): %g\n", average);


    /*
     * Repeat the above steps to load all pixels as a double
     * and an int array, and check the results.
     */

    printf("Loading pixel buffer as a double array...\n");
    /* Set load structure */
    ql.ptype = PTYPE_DOUBLE ;
    if (qfitsloader_init(&ql)!=0) {
        return -1 ;
    }
    /* Perform load */
    if (qfits_loadpix(&ql)!=0) {
        return -1 ;
    }
    /* Compute the average pixel value */
    average=0 ;
    for (i=0 ; i<npix ; average += ql.dbuf[i++]) ;
    average /= (double)npix ;
    free(ql.dbuf);
	printf("Average pixel value (double): %g\n", average);


    printf("Loading pixel buffer as an int array...\n");
    /* Set load structure */
    ql.ptype = PTYPE_INT ;
    if (qfitsloader_init(&ql)!=0) {
        return -1 ;
    }
    /* Perform load */
    if (qfits_loadpix(&ql)!=0) {
        return -1 ;
    }
    /* Compute the average pixel value */
    average=0 ;
    for (i=0 ; i<npix ; average += (double)ql.ibuf[i++]) ;
    average /= (double)npix ;
    free(ql.ibuf);
	printf("Average pixel value (int): %g\n", average);

    /* Call xmemory_status to check for memory leaks */
    xmemory_status();
	return 0 ;
}

